@@ -69,7 +69,7 @@ ga_spCrossover <- function(object, parents, ...)
   fitness <- object@fitness[parents]
   parents <- object@population[parents,,drop = FALSE]
   n <- ncol(parents)
-  children <- matrix(NA, nrow = 2, ncol = n)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
   fitnessChildren <- rep(NA, 2)
   crossOverPoint <- sample(0:n, size = 1)
   if(crossOverPoint == 0)
@@ -83,7 +83,7 @@ ga_spCrossover <- function(object, parents, ...)
                            parents[2,(crossOverPoint+1):n])
            children[2,] <- c(parents[2,1:crossOverPoint],
                            parents[1,(crossOverPoint+1):n])
-           fitnessChildren <- NA }
+         }
   out <- list(children = children, fitness = fitnessChildren)
   return(out)
 }
@@ -95,7 +95,9 @@ ga_spCrossover <- function(object, parents, ...)
 gabin_Population <- function(object, ...)
 {
 # Generate a random population of nBits 0/1 values of size popSize
-  population <- matrix(NA, nrow = object@popSize, ncol = object@nBits)
+  population <- matrix(as.double(NA), 
+                       nrow = object@popSize, 
+                       ncol = object@nBits)
   for(j in 1:object@nBits) 
      { population[,j] <- round(runif(object@popSize)) }
   return(population)
@@ -113,13 +115,13 @@ gabin_spCrossover <- ga_spCrossover
 
 gabin_uCrossover <- function(object, parents, ...)
 {
-# Uniform crossover - da rivedere come in mclustgasel
+# Uniform crossover
   parents <- object@population[parents,,drop = FALSE]
   n <- ncol(parents)
   u <- runif(n)
   children <- parents
-  children[,u > 0.5] <- abs(children[,u > 0.5]-1)
-  out <- list(children = children, fitness = NA)
+  children[1:2, u > 0.5] <- children[2:1, u > 0.5]
+  out <- list(children = children, fitness = rep(NA,2))  
   return(out)
 }
 
@@ -144,7 +146,7 @@ gareal_Population <- function(object, ...)
   min <- object@min
   max <- object@max
   nvars <- length(min)
-  population <- matrix(NA, nrow = object@popSize, ncol = nvars)
+  population <- matrix(as.double(NA), nrow = object@popSize, ncol = nvars)
   for(j in 1:nvars) 
      { population[,j] <- runif(object@popSize, min[j], max[j]) }
   return(population)
@@ -218,12 +220,11 @@ gareal_waCrossover <- function(object, parents, ...)
 # Whole arithmetic crossover
   parents <- object@population[parents,,drop = FALSE]
   n <- ncol(parents)
-  children <- matrix(NA, nrow = 2, ncol = n)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
   a <- runif(1)
   children[1,] <- a*parents[1,] + (1-a)*parents[2,]
   children[2,] <- a*parents[2,] + (1-a)*parents[1,]
-  fitness <- NA         
-  out <- list(children = children, fitness = fitness)
+  out <- list(children = children, fitness = rep(NA,2))
   return(out)
 }
 
@@ -232,12 +233,11 @@ gareal_laCrossover <- function(object, parents, ...)
 # Local arithmetic crossover
   parents <- object@population[parents,,drop = FALSE]
   n <- ncol(parents)
-  children <- matrix(NA, nrow = 2, ncol = n)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
   a <- runif(n)
   children[1,] <- a*parents[1,] + (1-a)*parents[2,]
   children[2,] <- a*parents[2,] + (1-a)*parents[1,]
-  fitness <- NA         
-  out <- list(children = children, fitness = fitness)
+  out <- list(children = children, fitness = rep(NA,2))
   return(out)
 }
 
@@ -248,17 +248,54 @@ gareal_blxCrossover <- function(object, parents, ...)
   n <- ncol(parents)
   a <- 0.5
   # a <- exp(-pi*iter/max(iter)) # annealing factor
-  children <- matrix(NA, nrow = 2, ncol = n)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
   for(i in 1:n)
      { x <- sort(parents[,i])
        xl <- max(x[1] - a*(x[2]-x[1]), object@min[i])
        xu <- min(x[2] + a*(x[2]-x[1]), object@max[i])
        children[,i] <- runif(2, xl, xu) 
      }
-  out <- list(children = children, fitness = NA)
+  out <- list(children = children, fitness = rep(NA,2))
   return(out)
 }
 
+# Laplace crossover(a, b)
+#
+# a is the location parameter and b > 0 is the scaling parameter of a Laplace
+# distribution, which is generated as described in 
+# Krishnamoorthy K. (2006) Handbook of Statistical Distributions with 
+#   Applications, Chapman & Hall/CRC.
+#
+# For smaller values of b offsprings are likely to be produced nearer to 
+# parents, and for larger values of b offsprings are expected to be produced
+# far from parents.
+# Deep et al. (2009) suggests to use a = 0, b = 0.15 for real-valued 
+# variables, and b = 0.35 for integer variables.
+#
+# References
+#
+# Deep K., Singh K.P., Kansal M.L., Mohan C. (2009) A real coded genetic
+#   algorithm for solving integer and mixed integer optimization problems.
+#   Applied Mathematics and Computation, 212(2), pp. 505-518.
+
+
+gareal_laplaceCrossover <- function (object, parents, a = 0, b = 0.35, ...) 
+{
+  parents <- object@population[parents, , drop = FALSE]
+  n <- ncol(parents)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
+  u <- runif(n)
+  beta <- a + ifelse(u > 0.5, 
+                     -b*log(2*(1 - u)), 
+                     +b*log(2*u))
+  bpar <- beta*abs(parents[1,] - parents[2,])
+  children[1,] <- pmin(pmax(parents[1,] + bpar, object@min), object@max)
+  children[2,] <- pmin(pmax(parents[2,] + bpar, object@min), object@max)
+  out <- list(children = children, fitness = rep(NA, 2))
+  return(out)
+}
+
+
 gareal_raMutation <- function(object, parent, ...)
 {
 # Uniform random mutation
@@ -299,6 +336,39 @@ gareal_rsMutation <- function(object, parent, ...)
   return(mutate)
 }
 
+# Power mutation(pow)
+#
+# a is the location parameter and b > 0 is the scaling parameter of a Laplace
+# distribution, which is generated as described in 
+# Krishnamoorthy K. (2006) Handbook of Statistical Distributions with 
+#   Applications, Chapman & Hall/CRC.
+#
+# For smaller values of b offsprings are likely to be produced nearer to 
+# parents, and for larger values of b offsprings are expected to be produced
+# far from parents.
+
+# Deep et al. (2009) suggests to use pow = 10 for real-valued variables, and
+# pow = 4 for integer variables.
+#
+# References
+#
+# Deep K., Singh K.P., Kansal M.L., Mohan C. (2009) A real coded genetic
+#   algorithm for solving integer and mixed integer optimization problems.
+#   Applied Mathematics and Computation, 212(2), pp. 505-518.
+
+gareal_powMutation <- function(object, parent, pow = 4, ...)
+{
+  mutate <- parent <- as.vector(object@population[parent,])
+  n <- length(parent)
+  s <- runif(1)^pow
+  t <- (parent - object@min)/(object@max - parent)
+  r <- runif(n)
+  mutate <- parent + ifelse(r > t, 
+                            +s*(object@max - parent), 
+                            -s*(parent - object@min))
+  return(mutate)
+}
+
 ## 
 ## Permutation GA operators 
 ##
@@ -308,7 +378,7 @@ gaperm_Population <- function(object, ...)
 # Generate a random permutation of size popSize in the range [min, max]  
   min <- object@min
   max <- object@max
-  population <- matrix(NA, nrow = object@popSize, ncol = max)
+  population <- matrix(as.double(NA), nrow = object@popSize, ncol = max)
   for(i in 1:object@popSize)
      population[i,] <- sample(min:max, size = max, replace = FALSE)
   return(population)
@@ -332,7 +402,7 @@ gaperm_cxCrossover <- function(object, parents, ...)
   children[1:2,cxPoint] <- parents[2:1,cxPoint]
   while( length(dup <- which(duplicated(children[1,], fromLast = TRUE))) > 0 )
        { children[1:2,dup] <- children[2:1,dup] }
-  out <- list(children = children, fitness = NA)
+  out <- list(children = children, fitness = rep(NA,2))
   return(out)
 }
 
@@ -343,7 +413,7 @@ gaperm_pmxCrossover <- function(object, parents, ...)
   n <- ncol(parents)
   cxPoints <- sample(1:n, size = 2)
   cxPoints <- seq(min(cxPoints), max(cxPoints))
-  children <- matrix(NA, nrow = 2, ncol = n)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
   children[,cxPoints] <- parents[,cxPoints]
   for(i in setdiff(1:n, cxPoints))
      { if(!any(parents[2,i] == children[1,cxPoints]))
@@ -353,7 +423,7 @@ gaperm_pmxCrossover <- function(object, parents, ...)
      }
   children[1,is.na(children[1,])] <- setdiff(parents[2,], children[1,])
   children[2,is.na(children[2,])] <- setdiff(parents[1,], children[2,])
-  out <- list(children = children, fitness = NA)
+  out <- list(children = children, fitness = rep(NA,2))
   return(out)
 }
 
@@ -365,7 +435,7 @@ gaperm_oxCrossover <- function(object, parents, ...)
   #
   cxPoints <- sample(seq(2,n-1), size = 2)
   cxPoints <- seq(min(cxPoints), max(cxPoints))
-  children <- matrix(NA, nrow = 2, ncol = n)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
   children[,cxPoints] <- parents[,cxPoints]
   #
   for(j in 1:2)
@@ -375,7 +445,7 @@ gaperm_oxCrossover <- function(object, parents, ...)
        children[j,i] <- val
      }
   #
-  out <- list(children = children, fitness = NA)
+  out <- list(children = children, fitness = rep(NA,2))
   return(out)
 }
 
@@ -386,7 +456,7 @@ gaperm_pbxCrossover <- function(object, parents, ...)
   n <- ncol(parents)
   #
   cxPoints <- unique(sample(1:n, size = n, replace = TRUE))
-  children <- matrix(NA, nrow = 2, ncol = n)
+  children <- matrix(as.double(NA), nrow = 2, ncol = n)
   children[1,cxPoints] <- parents[2,cxPoints]
   children[2,cxPoints] <- parents[1,cxPoints]
   #
@@ -396,7 +466,7 @@ gaperm_pbxCrossover <- function(object, parents, ...)
        children[j,pos] <- val
      }
   #
-  out <- list(children = children, fitness = NA)
+  out <- list(children = children, fitness = rep(NA,2))
   return(out)
 }
 
@@ -467,3 +537,48 @@ gaperm_scrMutation <- function(object, parent, ...)
   mutate <- parent[i]
   return(mutate)
 } 
+
+ga_pmutation <- function(object, p0 = 0.5, p = 0.01, 
+                         T = round(object@maxiter/2), ...)
+{
+# variable probability of mutation
+# p0 = initial pmutation
+# p = final pmutation
+# T = maximum iteration after which converges to p
+#
+# Example:
+# p0 = 0.5; p = 0.01; 
+# maxiter = 1000; T = round(maxiter/2); t = seq(maxiter)
+# pm1 = ifelse(t > T, p, p0 - (p0-p)/T * (t-1)) # linear decay
+# pm2 = (p0 - p)*exp(-2*(t-1)/T) + p # exponential decay
+# plot(t, pm1, type = "l")
+# lines(t, pm2, col = 2)
+
+  t <- object@iter
+  # linear decay
+  # pm <- if(t > T) p else p0 - (p0-p)/T * (t-1)
+  # exponential decay
+  pm = (p0 - p)*exp(-2*(t-1)/T) + p
+  #
+  return(pm)
+}
+
+# Probability of selection based on fitness values in vector x with
+# selection pressure given by q in (0,1).
+# This is used in optim() local search to select which solution should
+# be used for starting the algorithm.
+optimProbsel <- function(x, q = 0.25)
+{
+  x <- as.vector(x)
+  n <- length(x)
+  # selection pressure parameter
+  q <- min(max(sqrt(.Machine$double.eps), q), 
+           1 - sqrt(.Machine$double.eps))
+  rank <- (n + 1) - rank(x, ties.method = "random", na.last = FALSE)
+  # prob <- q*(1-q)^(rank-1) * 1/(1-(1-q)^n)
+  prob <- q*(1-q)^(rank-1)
+  prob[is.na(x)] <- 0
+  prob <- prob/sum(prob)
+  return(prob)
+}
+  
\ No newline at end of file
